/*
** Module   : MasterPty.java
** Abstract : an abstract interface with native PTY.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ----------------Description-----------------
** 001 MAG 20140318 First version.
*/

package com.goldencode.terminal;

import java.io.*;

/**
 * This class is a simplified interface with PTY native code.
 * It provide buffered read operations.
 * 
 * @author mag
 *
 */
public abstract class MasterPty 
extends NativePty
{
   /** TERM environment variable template */
   private static final String TERM = "TERM=%s";
         
   /** Read buffer */
   private ByteArrayInputStream buffer = null;
   
   /** Used to convert an UTF-16 character to an UTF-8 byte array */
   private char[] utf16 = new char[1];   
         
   /**
    * Constructor.
    * 
    * @param   term
    *          Terminal type. 
    */
   public MasterPty(String term, String[] cmdLine)
   {
      super();
      this.sh(term, cmdLine);
   }
   
   /**
    * Run shell.
    * 
    * @param   term
    *          Terminal environment example vt100
    * @param   cmdLine
    *          Command line arguments
    */
   public int sh(String term, String[] cmdLine)
   {
      pid = shell(String.format(MasterPty.TERM, term), cmdLine);
      return pid;
   }
         
   /**
    * Wait for child process to exit.
    * 
    * @return  Process status or -1 on error.
    */
   public int await()
   {
      return waitPid(pid);
   }
   
   /**
    * Read from master PTY.
    * 
    * @return  A byte array read from master PTY or an empty array if no characters 
    *          are available. 
    */
   public byte[] get()
   {
      return read(mfd_inp);
   }

   /**
    * Write to master PTY.
    * 
    * @param   data
    *          Array of bytes to write.
    *          
    * @return  Number of written bytes. 
    */
   public int put(byte[] data)
   {
      // Write data
      return write(mfd_out, data);
   }   

   /**
    * Write to master PTY.
    * 
    * @param   data
    *          String to write.
    *          
    * @return  Number of written bytes. 
    */
   public int put(String data)
   {
      return put(data.getBytes());
   }   

   /**
    * Convert UTF-16 character to an UTF-8 byte array and write it to master PTY. 
    * 
    * @param   ch
    *          character to write.
    *          
    * @return  Number of written bytes. 
    */
   public int put(char ch)
   {
      int count = -1;

      // store character      
      utf16[0] = ch;
      
      // convert and write to master PTY
      try
      {
         count = put(new String(utf16).getBytes("UTF-8"));
      }
      catch (UnsupportedEncodingException e)
      {
         e.printStackTrace();
      };
      
      // return number of written bytes. 
      return count;
   }      
   
   /**
    * Buffered read next byte from master PTY.
    * 
    * @return  Next byte from master PTY 0r -1 if buffer is empty.
    */
   protected byte nextByte()
   {
      if (buffer == null || buffer.available() == 0)
      {
         byte[] data = this.get();    
         buffer = new ByteArrayInputStream(data);
      }

      return (byte) buffer.read();      
   }
   
   /**
    * Get child process id.
    * 
    * @return  child process id.
    */
   public int getPid()
   {
      return this.pid;
   }
   
   /**
    * Wait for child process to exit.
    */
   public void waitFor()
   {
      this.await();
   }
}
