/*
** Module   : VT100Renderer.java
** Abstract : VT-100 renderer contract.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ----------------Description-----------------
** 001 MAG 20140318 First version.
*/

package com.goldencode.terminal;

/**
 * VT-100 renderer contract.
 * 
 * @author mag
 *
 */
public interface VT100Renderer
{   
   /**
    * Set cursor position.
    *  
    * @param   row
    *          Line number.
    * @param   col
    *          Column number.
    */
   public void setCursorPos(int row, int col);

   /**
    * Move cursor position with the amount of rows and columns.
    *  
    * @param   nRows
    *          Number of rows.
    * @param   nCols
    *          Number of columns.
    */
   public void moveCursorPos(int nRows, int nCols);

   /**
    * Set cell text attributes and color.
    * 
    * @param   text
    *          Character that should be displayed.
    * @param   attribute
    *          Cell attributes.
    * @param   color
    *          Call color FG/BG
    */
   public void setCellText(char text, int attribute, int color);
   
   /**
    * Play a message beep.
    */   
   public void beep();
   
   /**
    * Clear lines on screen.
    * 
    * @param   mode
    *          An integer value representing a screen region:
    *          0 - Erase from the active position to the end of the screen, inclusive (default)
    *          1 - Erase from start of the screen to the active position, inclusive
    *          2 - Erase all of the display -- all lines are erased, changed to single-width, 
    *              and the cursor does not move.
    */
   public void eraseInDisplay(int mode);   

   /**
    * Clear characters in current line.
    * 
    * @param   mode
    *          An integer value representing a line region:
    *          0 - Erase from the active position to the end of the line, inclusive (default)
    *          1 - Erase from the start of the line to the active position, inclusive
    *          2 - Erase all of the line, inclusive
    */
   public void eraseInLine(int mode);   
   
   /**
    * Set cursor at column.
    * 
    * @param   column
    *          Column number.
    */
   public void setCursorCol(int column);
   
   /**
    * NEL -- Next Line ESC E
    * 
    * This sequence causes the active position to move to the first position on the next line
    * downward. If the active position is at the bottom margin, a scroll up is performed.
    */
   public void nextLine();
   
   /**
    * IND -- Index ESC D
    * 
    * This sequence causes the active position to move downward one line without changing the 
    * column position. If the active position is at the bottom margin, a scroll up is performed.    
    */
   public void index();

   /**
    * RI -- Reverse Index ESC M
    * 
    * Move the active position to the same horizontal position on the preceding line.
    * If the active position is at the top margin, a scroll down is performed.
    */
   public void reverseIndex();

   /**
    * Set cursor visibility.
    * 
    * @param   showCursor
    * 
    */
   public void setCursorVisible(boolean showCursor);

   /**
    * Set auto wrap mode.
    * 
    * @param   autoWrap
    *          
    */
   public void setAutoWrap(boolean autoWrap);
      
   /**
    * Set scroll window.
    * 
    * @param   begLine
    *          Start line.
    * @param   endLine
    *          End line.
    */
   public void setScrollRegion(int begLine, int endLine);
   
   /**
    * Get cursor current row position.
    *  
    * @return  row number
    */
   public int getCursorRow();
   
   /**
    * Get cursor current column position.
    * 
    * @return  column number
    */
   public int getCursorCol();

   /**
    * Get number of columns.
    * 
    * @return  Number of columns
    */
   public int getColumns();
   
   /**
    * Get attribute mask.
    * 
    * @param   id
    *          Attribute id.
    * <p>
    *          0 - NORMAL 
    *          1 - BLINK 
    *          2 - BOLD 
    *          3 - UNDERLINE 
    *          4 - REVERSE
    *          
    * @return  attribute mask.
    */
   public int attributeMask(int id);
}
