/*
** Module   : shell_win.c
** Abstract : Run WINDOWS shell "cmd.exe" and control STDIN, STDOUT and STDERR file descriptors.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 MAG 20140318 Created initial version.
*/

#include <windows.h>
#include <stdio.h>

/**
 * Start a WINDOWS shell session. Simulate a Linux PTY using pipes. 
 * 
 * @param   term
 *          A string to set terminal type environment.
 *          Not used on Windows OS.
 * @param   cmdline
 *          Command line arguments.
 * @param   mfd_inp
 *          Pointer to a integer which store the STDIN handle.
 * @param   mfd_out
 *          Pointer to a integer which store the STDOUT, STDERR handles.
 *
 * @return  On success return process handle and set standard files handles.
 *          On error, -1 is returned, and errno is set appropriately.
 */
int os_shell(char* terminal, char** argv, int* mfd_inp, int *mfd_out) 
{
   // command line
   char cmd[8192] = "";
   
   DWORD dwCreationFlags = CREATE_NO_WINDOW;
   HANDLE hReadInp = 0, hWriteInp = 0;
   HANDLE hReadOut = 0, hWriteOut = 0;
   
   STARTUPINFO         si = {0};
   PROCESS_INFORMATION pi = {0};
   
   SECURITY_ATTRIBUTES sa = {0}; 
   sa.nLength = sizeof(SECURITY_ATTRIBUTES); 
   sa.bInheritHandle = TRUE; 
   sa.lpSecurityDescriptor = NULL; 

   // Indicate if pipes are created and configured
   int pipesOk = 0;
   
   // Create pipe for STDIN
   if (CreatePipe(&hReadInp, &hWriteInp, &sa, 0))
   {
      if (SetHandleInformation(hWriteInp, HANDLE_FLAG_INHERIT, 0))
      {
         // Create pipe for STDOUT and STDERR
         if (CreatePipe(&hReadOut, &hWriteOut, &sa, 0))
         {
            if (SetHandleInformation(hReadOut, HANDLE_FLAG_INHERIT, 0))
            {
               // pipes are created
               pipesOk = 1;
               
               // Prepare to start process
               si.cb = sizeof(STARTUPINFO);
               si.dwFlags |= STARTF_USESTDHANDLES;
               
               // Standard files redirection
               si.hStdInput = hReadInp;
               si.hStdOutput =  hWriteOut;
               si.hStdError = hWriteOut;
               
               // Build command line
               if (argv)
               {   
                  char* arg = *argv;
                  while (arg)
                  {
                     if (cmd[0] != 0)
                        strcat(cmd, " ");
                     strcat(cmd, arg);
                     argv++;
                     arg = *argv;
                  }
               }
            }
         }
      }   
   }
      
   // Execute command
   if (pipesOk && CreateProcess(NULL,             // application name
                                cmd,              // command line
                                NULL,             // process security attributes
                                NULL,             // thread security attribute
                                TRUE,             // inherit handles
                                dwCreationFlags,  // flags
                                NULL,             // environment block
                                NULL,             // current directory
                                &si,              // startup info
                                &pi))             // process info
   {
      // Pipe handles
      *mfd_inp = HandleToLong(hReadOut);
      *mfd_out = HandleToLong(hWriteInp);
      // Close unused parent handles
      CloseHandle(pi.hThread);
      CloseHandle(hReadInp);
      CloseHandle(hWriteOut);
      // Return pid
      return HandleToLong(pi.hProcess);
   }
   else
   {
      // Clean up
      CloseHandle(hReadInp);
      CloseHandle(hWriteInp);
      CloseHandle(hReadOut);
      CloseHandle(hWriteOut);
   }

   // On error return -1
   return -1;
}

/**
 * Wait for a child process to exit.
 * 
 * @param   pid
 *          Child process handle.
 *
 * @return  Process status.
 *          On error, a non zero value is returned, and errno is set appropriately.
 */
int wait_pid(int pid)
{
   HANDLE hProcess = LongToHandle(pid);
   // Wait for the process to finish with completion I/O operations
   WaitForSingleObjectEx(hProcess, INFINITE, TRUE); 
   return CloseHandle(hProcess);
}

/**
 * Read from master PTY file descriptor.
 * 
 * @param   mfd
 *          STDOUT, STDERR handle.
 * @param   buffer
 *          The read buffer.
 * @param   size
 *          Maximum size to read.
 *
 * @return  Number of bytes read or -1 on error. 
 */
int read_pty(int mfd, char* buffer, int size)
{
   DWORD dwAvail = 0, nNumberOfBytesRead = -1;
   HANDLE hPipe = LongToHandle(mfd);
   if (PeekNamedPipe(hPipe, NULL, 0, NULL, &dwAvail, NULL))
   {
      if (dwAvail)
      {   
         ReadFile(hPipe, buffer, size, &nNumberOfBytesRead, NULL);
      }
   }
   return nNumberOfBytesRead;
}

/**
 * Write to master PTY file.
 * 
 * @param   mfd
 *          STDINP handle.
 * @param   buffer
 *          Data to write.
 * @param   size
 *          Size of data to write.
 *
 * @return  Number of bytes written. 
 */
int write_pty(int mfd, void* buffer, int size)
{
   DWORD nNumberOfBytesWritten = -1;
   HANDLE hPipe = LongToHandle(mfd);
   WriteFile(hPipe, buffer, size, &nNumberOfBytesWritten, NULL);
   FlushFileBuffers(hPipe);
   return nNumberOfBytesWritten;
}
